@@ -3,6 +3,7 @@ import numpy as np
 import pathlib as pl
 import textwrap
 import xarray as xr
+import json
 # import os
 # import io
 
@@ -95,12 +96,20 @@ class MeasurementTree:
                 self.save_path = self.path.with_name(name).with_suffix('.nc').absolute()
                 self.array = xr.open_dataarray(self.save_path)
                 self.dataset = xr.open_dataarray(self.save_path)
-                return self.array
             except FileNotFoundError:
                 self.save_netcdf()
                 self.array = xr.open_dataarray(self.save_path)
                 self.dataset = xr.open_dataarray(self.save_path)
-                return self.array
+        try:
+            self.devices = json.loads(self.dataset.attrs['devices'])
+            self.labbook = json.loads(self.dataset.attrs['labbook'])
+            self.logs = json.loads(self.dataset.attrs['logs'])
+            self.definition = json.loads(self.dataset.attrs['scan_definition'])
+            self.tree_string = json.loads(self.dataset.attrs['measurement_tree'])
+        except KeyError:
+            print('Metadata not found. It will not be available.')
+
+
 
     def construct_tree(self):
         self.definition = {i: self.convert_to_dict(k) for (i, k) in self.f['scan_definition'].items()}
@@ -426,6 +435,17 @@ class MeasurementTree:
 
         self.dataset = xr.combine_by_coords(all_indicators)
 
+        # add metadata to xarray attrs
+        attrs = {'devices': self.devices, 'labbook': self.labbook, 'logs': self.logs, 'measurement_tree': self.tree_string, 'scan_definition': self.definition}
+        from json import dumps
+        for attr, val in attrs.items():
+            self.dataset.attrs[attr] = dumps(val)
+            # if isinstance(val, dict):
+            #     self.dataset.attrs[attr] = dumps(val)
+            # elif isinstance(val, str):
+            #     self.dataset.attrs[attr] = dumps(val)
+
+
     @staticmethod
     def avoid_duplicate(init_name, lis):
         itera = 1
@@ -591,26 +611,36 @@ def consolidate_dims(array, name_includes, compare_to: str or None = None, new_d
                 candidates.append(x)
     elif isinstance(name_includes, list):
         candidates = name_includes
-    # initialize compare to
-    if compare_to is None:
-        compare_to = candidates[0]
-    elif compare_to in candidates:
-        pass
-    else:
-        raise KeyError("compare_to argument not found in selected dimensions")
-    candidates.remove(compare_to)
-    # initialize new_dim
-    if new_dim is None:
-        new_dim = compare_to
-    # initialize list of identical candidates
-    identicals_list = [compare_to]
-
-    temp = array[compare_to].data
-    # check if all axis are identical to temp
-    for dim_name in candidates:
-        if np.array_equal(array[dim_name].data, temp):
-            identicals_list.append(dim_name)
-    print(f'Duplicate coordinates found:\n{identicals_list}')
+    candidates_backup = candidates.copy()
+    compare_to_backup = compare_to
+    new_dim_backup = new_dim
+    for test in candidates_backup:
+        candidates = candidates_backup.copy()
+        compare_to = compare_to_backup
+        new_dim = new_dim_backup
+        # initialize compare to
+        if compare_to is None:
+            compare_to = test
+        elif compare_to in candidates:
+            pass
+        else:
+            raise KeyError("compare_to argument not found in selected dimensions")
+        candidates.remove(compare_to)
+        # initialize new_dim
+        if new_dim is None:
+            new_dim = compare_to
+        # initialize list of identical candidates
+        identicals_list = [compare_to]
+        temp = array[compare_to].data
+        # check if all axis are identical to temp
+        for dim_name in candidates:
+            if np.array_equal(array[dim_name].data, temp):
+                identicals_list.append(dim_name)
+        if len(identicals_list)>1:
+            print(f'Duplicate coordinates found:\n{identicals_list}')
+            break
+    if not len(identicals_list)>1:
+        raise ValueError("No duplicate coordinate axis found. Try using the 'compare_to' argument.")
     from string import ascii_letters, digits
     from random import choices
     temp_name = "".join(choices(ascii_letters+digits, k=16))
